#===============================================
#
#   flags.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package flags;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
FLAGS_get_flags
);
}




use glo::env;
use gbsfile;




sub FLAGS_get_flags($);

sub read_flags($);




my %FLAGS_CACHE;






sub FLAGS_get_flags($)
{
my ($filespec) = @_;

return read_flags( $filespec);
}




sub read_flags($)
{
my ($filespec) = @_;
my @lines;

if (exists( $FLAGS_CACHE{$filespec}))
{

@lines = @{$FLAGS_CACHE{$filespec}};
} else
{
GBSFILE_open( $filespec, 1, undef, undef);
my $line = GBSFILE_get_line();
while ($line)
{
$line = ENV_expand_envs( $line);
if (ENV_is_option( $line))
{
my $opt = substr( $line, 0, 2);
GBSFILE_sig( EE => "'$opt' flags not allowed in GBS FLAG Files")
if (substr( $opt , 1, 1) eq 'I' || substr( $opt , 1, 1) eq 'L')
}
push @lines, $line;
$line = GBSFILE_get_line();
}
$FLAGS_CACHE{$filespec} = [ @lines ];
}

return @lines;
}

1;

