#===============================================================
#
#   fspec.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package fspec;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
FSPEC_gen_type
FSPEC_sca
FSPEC_export
FSPEC_att_file
FSPEC_scope
FSPEC_owners
FSPEC_system
FSPEC_steps
FSPEC_switch
FSPEC_sys
FSPEC_subsys_script
FSPEC_makefile
FSPEC_ssprop
FSPEC_scadef
FSPEC_gbsdb
FSPEC_gbsall
FSPEC_gbsmake_file
);
}




use glo::env;
use gbsenv;




sub FSPEC_gen_type($$$$$);
sub FSPEC_sca($$$$$);
sub FSPEC_export($$$$);
sub FSPEC_att_file($$$);
sub FSPEC_scope($$$$);
sub FSPEC_owners($$);
sub FSPEC_system($$);
sub FSPEC_steps($$);
sub FSPEC_switch($$);
sub FSPEC_sys($$);
sub FSPEC_subsys_script($$$);
sub FSPEC_makefile($$);
sub FSPEC_ssprop($$);
sub FSPEC_scadef($);
sub FSPEC_gbsdb();
sub FSPEC_gbsall($$);
sub FSPEC_gbsmake_file($$);

sub get_filetype($$);




my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();
my $COMMAND_FILE_EXT = ENV_shell_filetype();








sub FSPEC_gen_type($$$$$)
{
my ($file,		    # incs_<type>.gbs, incs_<type>.usr, etc. see above
$root_path,
$subsys,
$component,
$target,
) = @_;



my $dir = $root_path;
if ($subsys)
{
$dir .= "/dev/$subsys";
if ($component)
{
$dir .= "/comp/$component/opt";
} else
{
$dir .= '/build';
}
} else
{
$dir .= '/sysbuild';
}
if ($target)
{
$dir .= "/$target";
}


return (wantarray) ? ($dir, $file) : "$dir/$file";
}






sub FSPEC_sca($$$$$)
{
my ($file,		    # sca_<target>, etc. see above
$root_path,
$subsys,	    # may be ''
$audit,
$target_or_name,    # may also be <uppercase_name> (not used)
) = @_;



my $dir;
if ($subsys)
{
$dir = "$root_path/dev/$subsys/audit/$audit";
} else
{
$dir = "$root_path/sysaudit/$audit";
}

return (wantarray) ? ($dir, $file) : "$dir/$file";
}





sub FSPEC_export($$$$)
{
my ($file,		    # with or without .gbs or .usr
$root_path,
$subsys,
$component,
) = @_;



my $dir;
if (!defined $component)
{
$dir = "$root_path/dev/$subsys";
} elsif ($component eq '')
{
$dir = "$root_path/dev/$subsys/build";
} else
{
$dir = "$root_path/dev/$subsys/comp/$component";
}

return get_filetype( $dir, $file);
}







sub FSPEC_att_file($$$)
{
my ($file,			# e.g. target.gbs, audit.usr, target
$root_path,
$audit_target_tool,	# e.g. $GBS_TARGET
) = @_;



my $dir;
{
my $file_name = ENV_split_spec_n( $file);
if ($file_name eq 'target')
{
$dir = "$root_path/sysbuild/$audit_target_tool";
} elsif ($file_name eq 'audit')
{
$dir = "$root_path/sysaudit/$audit_target_tool";
} elsif ($file_name eq 'tool')
{
$dir = "$root_path/systools/$audit_target_tool";
} else
{
ENV_sig( F => "Invalid att_file '$file', '$root_path', '$audit_target_tool'");
}
}

return get_filetype( $dir, $file);
}





sub FSPEC_scope($$$$)
{
my ($file,		    # 'scope' with or without .gbs or .usr
$root_path,
$subsys,
$component,
) = @_;



return get_filetype( "$root_path/dev/$subsys/comp/$component", $file);
}




sub FSPEC_owners($$)
{
my ($file,		    #  'owners' with or without .gbs or .usr
$root_path,
) = @_;



return get_filetype( "$root_path/sys", $file);
}




sub FSPEC_system($$)
{
my ($file,		    # 'system', with or without .gbs or .usr
$root_path,
) = @_;



return get_filetype( $root_path, $file);
}





sub FSPEC_steps($$)
{
my ($file,		    # 'steps', with or without .gbs or .usr
$root_path,
) = @_;



return get_filetype( "$root_path/sys", $file);
}






sub FSPEC_switch($$)
{
my ($file,			# switch or


$root_path,
) = @_;



my $dir = $root_path;
$file .= (-e "$dir/$file.usr") ? '.usr' : '.gbs'
if ($file eq 'switch');
$file .= $COMMAND_FILE_EXT
if ($file !~ /(\.sh$)|(\.bat$)/);

return (wantarray) ? ($dir, $file) : "$dir/$file";
}





sub FSPEC_sys($$)
{
my ($file,		    # 'sys', with or without .gbs or .usr
$root_path,
) = @_;



return get_filetype( "$root_path/sys", $file);
}






sub FSPEC_subsys_script($$$)
{
my ($file,		   # 'gbssub' with or without .sh or .bat
$root_path,
$subsys,
) = @_;



my $dir = "$root_path/dev/$subsys";
$file .= $COMMAND_FILE_EXT
if ($file !~ /(\.sh$)|(\.bat$)/);

return (wantarray) ? ($dir, $file) : "$dir/$file";
}




sub FSPEC_makefile($$)
{
my ($root_path,
$subsys,
) = @_;



my $dir = "$root_path/dev/$subsys";
my $file = 'Makefile.mk';

return (wantarray) ? ($dir, $file) : "$dir/$file";
}





sub FSPEC_ssprop($$)
{
my ($root_path,
$subsys,
) = @_;



my $dir = "$root_path/dev/$subsys";
my $file = 'ssprop.gbs';

return (wantarray) ? ($dir, $file) : "$dir/$file";
}





sub FSPEC_scadef($)
{
my ($plugin_name,
) = @_;


return "$GBS_SCRIPTS_PATH/plugins/audit/$plugin_name/scadef.gbs";
}




sub FSPEC_gbsdb()
{



my $gbs_base_dir = ENV_getenv_perl_path( 'GBS_BASE_PATH');
my $file = 'gbsdb.dat';

return (wantarray) ? ($gbs_base_dir, $file) : "$gbs_base_dir/$file";
}




sub FSPEC_gbsall($$)
{
my ($filetype,	    # full_name+type or .sh or .bat or ''
$in_install,	    # In GBS_SCRIPTS_ROOT else in GBS_BASE_PATH
) = @_;



my $path = ($in_install) ? ENV_getenv_perl_path( 'GBS_SCRIPTS_ROOT') : ENV_getenv_perl_path( 'GBS_BASE_PATH');
my $file;

my ($name, $type) = ENV_split_spec_nt( $filetype);
ENV_sig( F => "Filename in '$filetype' must be 'gbsall'")
if ($name ne '' && $name ne 'gbsall');
if ($type eq '')
{
$file = "gbsall$COMMAND_FILE_EXT";
} else
{
ENV_sig( F => "Filetype in '$filetype' must be .bat or .sh")
if ($type ne '.bat' && $type ne '.sh');
$file = "gbsall$type";
}

return (wantarray) ? ($path, $file) : "$path/$file";
}




sub FSPEC_gbsmake_file($$)
{
my ($gbsgen_path,
$gbstarget,
) = @_;

return "$gbsgen_path/$gbstarget.mk";
}




sub get_filetype($$)
{
my ($dir,
$file,	    # type == .gbs or .usr or ''
) = @_;

if (index( $file, '.') == -1)
{
$file .= (-e "$dir/$file.usr") ? '.usr' : '.gbs';
}

return (wantarray) ? ($dir, $file) : "$dir/$file";
}

1;

