#===============================================================
#
#   gbsbgexec.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSBGEXEC @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::args;
use glo::spit;
use gbsenv;
use glkb;
use gbsbgglo;




sub do_glkb($$$$@);
sub get_command_args();




my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');




my $BATCH_NAME = '';	# reference to be used in logging. E.g.: gbsgen, gbsgen_post, gbsaudit, gbsaudit_post
my $COMPONENT;
my $GBSBG_SCRIPT_SPEC;	# main input-file






$| = 1;
my $RC = 0;
GBSENV_init();

get_command_args();

{
GBSBGGLO_init( $BATCH_NAME, $COMPONENT);    # sets CWD to SRC or SUSBSYS
$RC = GBSBGGLO_main( $GBSBG_SCRIPT_SPEC, [ GLKB => \&do_glkb ]);
}

ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 0);
}




sub do_glkb($$$$@)
{
my ($file,
$must_print_commands,
$glkb_args,	    # "$glkb_type $glkb_file $out_filespec $glkb_via_format"
$out_files,	    # List
@command_lines,
) = @_;
my $rc = 0;

ENV_whisper( 1, "GLKB $file");

my ($glkb_type, $glkb_file, $out_filespec, $glkb_via_format) = split( ' ', $glkb_args, 4);

my $gbs_component = ENV_getenv( 'GBS_COMPONENT');




GLKB_parse( $glkb_file, $GBS_TARGET, $gbs_component);
my ( $nr_errors, $nr_warnings, @gl_lines ) = GLKB_get_lines();
ENV_say( 1, "$nr_warnings warning(s) found while processing '$gbs_component:$glkb_file'")
if ($nr_warnings > 0);
if ($nr_errors > 0)
{
ENV_say( 1, "$nr_errors error(s) found while processing '$gbs_component:$glkb_file'");
$rc = 2;
GBSBGGLO_handle_rc( $rc, $file, 'GLKB', $out_files);
} else
{
if (@gl_lines == 0)
{
ENV_say( 1, "No lines in '$gbs_component:$glkb_file'");
$rc = 2;
GBSBGGLO_handle_rc( $rc, $file, 'GLKB', $out_files);
} elsif (@gl_lines == 1 && $gl_lines[0] eq 'DUMMY')
{




ENV_say( 1, "Test not executed because of Target suppression",
"Dummy output file(s) generated");
SPIT_file( $out_filespec, ("$file: Test not executed because of Target suppression\n"));
foreach my $out_file (split( ' ', $out_files))
{
SPIT_file( $out_file, ());
}
ENV_say( 0, '-');
} else
{



my $envvar_value;
if ($glkb_type eq 'FILE')
{
foreach my $item (@gl_lines)
{
$item = ENV_join_quoted_space( $item);
}
my $glkb_filespec = "../bld/$GBS_TARGET/$glkb_file";
SPIT_file_nl( $glkb_filespec, \@gl_lines);
$envvar_value = sprintf( $glkb_via_format, $glkb_filespec);
} else	#$glkb_type eq 'VARIABLE'
{
$envvar_value = ENV_join_quoted_space( @gl_lines);
}
GBSBGGLO_set( GBS_GLKB => $envvar_value);




$rc = GBSBGGLO_exec( $file, $must_print_commands, $out_files, @command_lines);
}
}

return $rc;
}




sub get_command_args()
{
($BATCH_NAME,
$COMPONENT,
$GBSBG_SCRIPT_SPEC,
) = ARGS_get( 3, 'perl -w gbsbgexec.pl <BATCH_NAME> <COMPONENT> <gbg_script>', undef);

}


