#===============================================================
#
#   gbsexec.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package gbsexec;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
GBSEXEC_gen
GBSEXEC_audit
);
}




use glo::env;
use gbsenv;
use cfo;
use bld;
use gbsbg;




sub GBSEXEC_gen($$);
sub GBSEXEC_audit($$$);

sub execute($$$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMP_PATH = "$GBS_ROOT_PATH/dev/$GBS_SUBSYS/comp";








my $OPTIONS_BANNER;





sub GBSEXEC_gen($$)
{
my ($nr_cfo,
$must_ignore_errors,
) = @_;
my $rc = 0;




$OPTIONS_BANNER = BLD_init_build();	# Also sets: GBS_BLD_*, GBS_MODE, GBS_OPT, etc.




$rc = execute( 'gbsgen', $nr_cfo, $must_ignore_errors);

return $rc;
}




sub GBSEXEC_audit($$$)
{
my ($audit,		    # can have multiple audit-runs in one execution of gbsaudit command
$nr_cfo,
$must_ignore_errors,
) = @_;
my $rc = 0;




$OPTIONS_BANNER = BLD_init_audit( $audit);	# Also sets: GBS_AUDIT and AUDIT_envs




$rc = execute( 'gbsaudit', $nr_cfo, $must_ignore_errors);

return $rc;
}




sub execute($$$)
{
my ($batch_name,		    # gbsgen gbsaudit
$nr_cfo,
$must_ignore_errors,	    # bool
) = @_;
my $rc = 0;

ENV_say( 1, "** $OPTIONS_BANNER");




my @pre_options = CFO_pre_options();




my $cur_order = 1;
for (my $i = 0; $i < $nr_cfo; $i++)
{



my ($sel_order, $component, $files_ref, $options_ref) = CFO_get_item( $i);
my @files = @{$files_ref};
my @options = @{$options_ref};




if ($sel_order != $cur_order)
{

my $this_rc = GBSBG_batch_wait();
$rc = $this_rc
if ($this_rc > $rc);
$cur_order = $sel_order;
last unless ($must_ignore_errors || $rc == 0);
}




ENV_chdir( "$GBS_COMP_PATH/$component/src");
my $is_queued;

{

my @lines;
foreach my $file (@files)
{

push @lines, BLD_get_commands( $component, $file, [ @pre_options, @options ]);
}
$is_queued = GBSBG_batch_queue( $batch_name, $component, \@lines);

}


last unless ($must_ignore_errors || $is_queued);
}





my $this_rc = GBSBG_batch_wait();
$rc = $this_rc
if ($this_rc > $rc);

return $rc;
}

1;

