#==============================================================
#
#   gbsglo.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package gbsglo;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
GBSGLO_reset
GBSGLO_system_audits
GBSGLO_system_builds
GBSGLO_system_tools
GBSGLO_subsystems
GBSGLO_subsystem_refs
GBSGLO_subsystem_exists
GBSGLO_subsystems_full_gbs
GBSGLO_subsystems_non_gbs
GBSGLO_subsystem_is_full_gbs
GBSGLO_subsystems_for_target
GBSGLO_subsystems_for_audit
GBSGLO_subsystems_for_tool
GBSGLO_subsystem_does_build
GBSGLO_subsystem_does_audit
GBSGLO_subsystem_does_tool
GBSGLO_subsystem_type
GBSGLO_subsystem_targets
GBSGLO_subsystem_audits
GBSGLO_subsystem_tools
GBSGLO_components
GBSGLO_component_exists
GBSGLO_components_for_target
GBSGLO_components_for_audit
GBSGLO_component_does_build
GBSGLO_component_does_audit
GBSGLO_component_targets
GBSGLO_component_audits
GBSGLO_glob
GBSGLO_validate_spec
GBSGLO_select_dirs
GBSGLO_select_specs
GBSGLO_exclude_specs
GBSGLO_short_filespecs
);
}




use glo::env;
use glo::slurp;
use gbsenv;
use ssprop;




sub GBSGLO_reset();
sub GBSGLO_system_audits();
sub GBSGLO_system_builds();
sub GBSGLO_system_tools();
sub GBSGLO_subsystems();
sub GBSGLO_subsystem_refs();
sub GBSGLO_subsystem_exists($$);
sub GBSGLO_subsystems_full_gbs();
sub GBSGLO_subsystems_non_gbs();
sub GBSGLO_subsystem_is_full_gbs($;$);
sub GBSGLO_subsystems_for_target($);
sub GBSGLO_subsystems_for_audit($$);
sub GBSGLO_subsystems_for_tool($);
sub GBSGLO_subsystem_does_build($$);
sub GBSGLO_subsystem_does_audit($$$);
sub GBSGLO_subsystem_does_tool($$);
sub GBSGLO_subsystem_type($);
sub GBSGLO_subsystem_targets($);
sub GBSGLO_subsystem_audits($);
sub GBSGLO_subsystem_tools($);
sub GBSGLO_components($);
sub GBSGLO_component_exists($$$);
sub GBSGLO_components_for_target($$);
sub GBSGLO_components_for_audit($$$);
sub GBSGLO_component_does_build($$$);
sub GBSGLO_component_does_audit($$$$);
sub GBSGLO_component_targets($$);
sub GBSGLO_component_audits($$);
sub GBSGLO_glob($);
sub GBSGLO_validate_spec($);
sub GBSGLO_select_dirs(@);
sub GBSGLO_select_specs(@);
sub GBSGLO_exclude_specs(@);
sub GBSGLO_short_filespecs(@);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_ROOT_QM = quotemeta( $GBS_ROOT_PATH);
my $GBS_SCRIPTS_PATH = GBSENV_scripts_path();
my $GBS_SCRIPTS_QM = quotemeta( $GBS_SCRIPTS_PATH);

my @GBS_SKIPTYPES = ENV_getenv( 'GBS_SKIPTYPES');



my @GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');
my %GBS_TARGETS = map { $_ => 1 } @GBS_TARGETS;

my @GBS_AUDITS = ENV_getenv( 'GBS_AUDITS');
my %GBS_AUDITS = map { $_ => 1 } @GBS_AUDITS;

my @GBS_TOOLS = ENV_getenv( 'GBS_TOOLS');
my %GBS_TOOLS = map { $_ => 1 } @GBS_TOOLS;




my @GBS_ALL_SUBSYSTEMS = ENV_getenv( 'GBS_ALL_SUBSYSTEMS');
my %GBS_ALL_SUBSYSTEMS = map { $_ => undef } @GBS_ALL_SUBSYSTEMS;





sub GBSGLO_reset()
{
@GBS_ALL_SUBSYSTEMS = ENV_getenv( 'GBS_ALL_SUBSYSTEMS');
%GBS_ALL_SUBSYSTEMS = map { $_ => undef } @GBS_ALL_SUBSYSTEMS;


SSPROP_reset();
}




sub GBSGLO_system_audits()
{
my @all_dirs;		# array or scalar (comma-list)

foreach my $spec (glob "$GBS_ROOT_PATH/sysaudit/*/audit.gbs")
{
my $struct = (split( '/', $spec))[-2]; # get the parent_dir name
push @all_dirs, $struct;
}

return (wantarray) ? @all_dirs : join( ',', @all_dirs);
}




sub GBSGLO_system_builds()
{
my @all_dirs;		# array or scalar (comma-list)

foreach my $spec (glob "$GBS_ROOT_PATH/sysbuild/*/target.gbs")
{
my $struct = (split( '/', $spec))[-2]; # get the parent_dir name
push @all_dirs, $struct;
}

return (wantarray) ? @all_dirs : join( ',', @all_dirs);
}




sub GBSGLO_system_tools()
{
my @all_dirs;		# array or scalar (comma-list)

foreach my $spec (glob "$GBS_ROOT_PATH/systools/*/tool.gbs")
{
my $struct = (split( '/', $spec))[-2]; # get the parent_dir name
push @all_dirs, $struct;
}

return (wantarray) ? @all_dirs : join( ',', @all_dirs);
}




sub GBSGLO_subsystems()
{
return @GBS_ALL_SUBSYSTEMS;
}




sub GBSGLO_subsystem_refs()
{
my @refs;	# [ $subsys, $ss_type ]

foreach my $subsys (@GBS_ALL_SUBSYSTEMS)
{
push @refs, [ $subsys, SSPROP_get_type( undef, $subsys) ];
}

return @refs;
}





sub GBSGLO_subsystem_exists($$)
{
my ($root_path,
$subsys,    #case sensitive!
) = @_;
my $exists;

if ($root_path eq $GBS_ROOT_PATH)
{
$exists = (exists $GBS_ALL_SUBSYSTEMS{$subsys}) ? 1 : 0;
} else
{
my %all_subsystems = map { $_ => 1 } GBSGLO_select_dirs( SLURP_dir_dirs( "$root_path/dev", 0));
$exists = (exists $all_subsystems{$subsys}) ? 1 : 0;
}


return $exists;
}




sub GBSGLO_subsystems_full_gbs()
{
return grep( SSPROP_is_full_gbs( undef, $_), GBSGLO_subsystems());
}




sub GBSGLO_subsystems_non_gbs()
{
return grep( ! SSPROP_is_full_gbs( undef, $_), GBSGLO_subsystems());
}




sub GBSGLO_subsystem_is_full_gbs($;$)
{
my ($subsys,
$root_path,	# optional. Use ONLY if $root_path ne $GBS_ROOT_PATH
) = @_;


return SSPROP_is_full_gbs( $root_path,  $subsys);
}




sub GBSGLO_subsystems_for_target($)
{
my ($target) = @_;


return grep( -e "$GBS_ROOT_PATH/dev/$_/build/$target", GBSGLO_subsystems());
}




sub GBSGLO_subsystems_for_audit($$)
{
my ($audit,
$target,    # may be undef
) = @_;


if (defined $target)
{
return grep( -e "$GBS_ROOT_PATH/dev/$_/audit/$audit/$target",  GBSGLO_subsystems());
} else
{
return grep( -e "$GBS_ROOT_PATH/dev/$_/audit/$audit",  GBSGLO_subsystems());
}
}




sub GBSGLO_subsystems_for_tool($)
{
my ($tool) = @_;


return grep( -e "$GBS_ROOT_PATH/dev/$_/tools/$tool", GBSGLO_subsystems());
}




sub GBSGLO_subsystem_does_build($$)
{
my ($subsys,
$target,
) = @_;

return (-e "$GBS_ROOT_PATH/dev/$subsys/build/$target") ? 1 : 0;
}




sub GBSGLO_subsystem_does_audit($$$)
{
my ($subsys,
$audit,
$target,
) = @_;

return (-e "$GBS_ROOT_PATH/dev/$subsys/audit/$audit/$target") ? 1 : 0;
}




sub GBSGLO_subsystem_does_tool($$)
{
my ($subsys,
$tool,
) = @_;

return (-e "$GBS_ROOT_PATH/dev/$subsys/tool/$tool") ? 1 : 0;
}




sub GBSGLO_subsystem_type($)
{
my ($subsys) = @_;
my $ss_type;	    # 'GBS', 'MSVS', 'make', 'Other'

$ss_type = SSPROP_get_type( undef, $subsys);

return $ss_type;
}





sub GBSGLO_subsystem_targets($)
{
my ($subsys) = @_;
my @targets;

@targets = SLURP_dir_dirs( "$GBS_ROOT_PATH/dev/$subsys/build", 0);
@targets = grep { exists $GBS_TARGETS{$_} } @targets;

return @targets;
}





sub GBSGLO_subsystem_audits($)
{
my ($subsys) = @_;
my @audits;

@audits = SLURP_dir_dirs( "$GBS_ROOT_PATH/dev/$subsys/audit", 0);
@audits = grep { exists $GBS_AUDITS{$_} } @audits;

return @audits;
}





sub GBSGLO_subsystem_tools($)
{
my ($subsys) = @_;
my @tools;

@tools = SLURP_dir_dirs( "$GBS_ROOT_PATH/dev/$subsys/tools", 0);
@tools = grep { exists $GBS_TOOLS{$_} } @tools;

return @tools;
}




sub GBSGLO_components($)
{
my ($subsys) = @_;


return GBSGLO_select_dirs( SLURP_dir_dirs( "$GBS_ROOT_PATH/dev/$subsys/comp", 0));
}





sub GBSGLO_component_exists($$$)
{
my ($root_path,
$subsys,    #case sensitive!
$component, #case sensitive!
) = @_;
my $exists;

if ($root_path eq $GBS_ROOT_PATH)
{
if (exists $GBS_ALL_SUBSYSTEMS{$subsys})
{
my $components_ref = $GBS_ALL_SUBSYSTEMS{$subsys};
if (!defined $components_ref)
{
my %all_components = map { $_ => 1 } GBSGLO_select_dirs( SLURP_dir_dirs( "$root_path/dev/$subsys/comp", 0));
$components_ref = \%all_components;
$GBS_ALL_SUBSYSTEMS{$subsys} = $components_ref;
}
$exists = (exists $components_ref->{$component}) ? 1 : 0;
} else
{
ENV_sig( F => "No such SubSystem '$root_path/dev/$subsys' for Component '$component'");
}
} else
{
my %all_components = map { $_ => 1 } GBSGLO_select_dirs( SLURP_dir_dirs( "$root_path/dev/$subsys/comp", 0));
$exists = (exists $all_components{$component}) ? 1 : 0;
}


return $exists;
}




sub GBSGLO_components_for_target($$)
{
my ($subsys,
$target,
) = @_;
my @components;

if (GBSGLO_subsystem_does_build( $subsys, $target))
{
@components = grep( -e "$GBS_ROOT_PATH/dev/$subsys/comp/$_/bld/$target", GBSGLO_components( $subsys));
}

return @components;
}




sub GBSGLO_components_for_audit($$$)
{
my ($subsys,
$audit,
$target,
) = @_;
my @components;

if (GBSGLO_subsystem_does_audit( $subsys, $audit, $target))
{
@components = grep( -e "$GBS_ROOT_PATH/dev/$subsys/comp/$_/aud/$audit/$target", GBSGLO_components( $subsys));
}

return @components;
}




sub GBSGLO_component_does_build($$$)
{
my ($subsys,
$component,
$target,
) = @_;

if (GBSGLO_subsystem_does_build( $subsys, $target))
{
return (-e "$GBS_ROOT_PATH/dev/$subsys/comp/$component/bld/$target") ? 1 : 0;
} else
{
return 0;
}
}




sub GBSGLO_component_does_audit($$$$)
{
my ($subsys,
$component,
$audit,
$target,
) = @_;

if (GBSGLO_subsystem_does_audit( $subsys, $audit, $target))
{
return (-e "$GBS_ROOT_PATH/dev/$subsys/comp/$component/aud/$audit/$target") ? 1 : 0;
} else
{
return 0;
}
}





sub GBSGLO_component_targets($$)
{
my ($subsys,
$component,
) = @_;
my @targets;

foreach my $target (@GBS_TARGETS)
{
push @targets, $target
if (GBSGLO_component_does_build( $subsys, $component, $target));
}

return @targets;
}




sub GBSGLO_component_audits($$)
{
my ($subsys,
$component,
) = @_;
my @audits;

foreach my $audit (@GBS_AUDITS)
{
foreach my $target (@GBS_TARGETS)
{
push @audits, $audit
if (GBSGLO_component_does_audit( $subsys, $component, $audit, $target));
}
}

return @audits;
}





sub GBSGLO_glob($)
{
my ($glob_string) = @_;	# May be a PATH
my @specs;

foreach my $spec (ENV_glob( $glob_string))
{
push @specs, $spec
unless (grep( $spec =~ $_, @GBS_SKIPTYPES));
}

return @specs;
}





sub GBSGLO_validate_spec($)
{
my ($spec) = @_;  	# path, dir, filespec or file


return (grep( $spec =~ $_, @GBS_SKIPTYPES)) ? 0 : 1;
}





sub GBSGLO_select_dirs(@)
{

my @out_dirs;

my $not_dir_chars_qm = quotemeta ':&()<>^;|,*? .';
@out_dirs = grep( $_ !~ /[$not_dir_chars_qm]/, @_);



return @out_dirs;
}





sub GBSGLO_select_specs(@)
{

my @out;


foreach my $spec (@_)
{
push @out, $spec unless (grep( $spec =~ $_, @GBS_SKIPTYPES));
}

return @out;
}





sub GBSGLO_exclude_specs(@)
{

my @out;


foreach my $file (@_)
{
push @out, $file if (grep( $file =~ $_, @GBS_SKIPTYPES));
}

return @out;
}




sub GBSGLO_short_filespecs(@)
{

my @short_filespecs;

foreach my $filespec (@_)
{
if ($filespec)
{
my $short_filespec = $filespec;
if (!($short_filespec =~ s/^$GBS_ROOT_QM/\$GBS_ROOT_PATH/))
{
$short_filespec =~ s/^$GBS_SCRIPTS_QM/\$GBS_SCRIPTS_PATH/
}
push @short_filespecs, $short_filespec;
} else
{
push @short_filespecs, $filespec
}
}

return (wantarray) ? @short_filespecs : $short_filespecs[0];
}

1;


