#==============================================================
#
#   gbsswt.pl
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
use strict;
use warnings;
use integer;
print "*GBSSWT @ARGV\n" if ($ENV{GBSDEBUG_FILE});




use lib $ENV{GBS_SCRIPTS_PATH};
use glo::env;
use glo::ask;
use glo::genopt;
use gbsenv;
use gbsglo;
use gbscmd;
use run;
use validate;
use swb;
use swglo;
use system;




sub ask_target();




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();
my $GBS_SUBSYS = ENV_getenv( 'GBS_SUBSYS');
my $GBS_COMPONENT = ENV_getenv( 'GBS_COMPONENT');
my $GBS_TARGET = ENV_getenv( 'GBS_TARGET');
my $GBS_AUDIT = ENV_getenv( 'GBS_AUDIT');

my $GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');	# commalist
my @GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');
@GBS_TARGETS = sort( @GBS_TARGETS);

my $CANCELLED = 0;






$| = 1;
my $RC = 0;
GBSENV_init();




my @genopts = (
[ '<1>',	'target',	     'sso',  '', "Target: '' == ASK, '-' == none, '.' == current" ],
[ 'new',	'create_new_target', 'bso',   0, "Create a new Target" ],
[ 'remove', 'remove_target',     'bso',   0, "Remove a Target from the Targets List" ],
);
my @genconflicts = (
[ [ new => 1 ], '=', [ remove => 1] ],
);
GENOPT_set_optdefs( [ 'gbsswt', 'swt' ], \@genopts,
'Set and/or Create new Current Target',
undef);
GENOPT_set_conflicts( \@genconflicts);
GENOPT_parse();
my $TARGET = GENOPT_get( 'target');
my $NEW = GENOPT_get( 'create_new_target');
my $REMOVE = GENOPT_get( 'remove_target');




VALIDATE_root();




if (!$NEW && @GBS_TARGETS == 0)
{
if (ASK_YN( "No Target(s) defined. Define now?", 'Y') eq 'Y')
{
$NEW = 1;
}
}




if ($NEW)
{
$TARGET = RUN_gbsnew( 'new_target', $TARGET);
@GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');
if ($TARGET eq '')
{
$TARGET = '.';
$CANCELLED = 1;
} else
{
ENV_say( 1, "Current Target is $TARGET");
}
} elsif ($REMOVE)
{
my $removed_target = RUN_gbsnew( 'rem_target', $TARGET);
@GBS_TARGETS = ENV_getenv( 'GBS_TARGETS');
if ($GBS_TARGET eq $removed_target)
{
ENV_say( 1, "Current Target removed");
$TARGET = '';	# No current
} else
{
ENV_say( 1, "Current Target is $GBS_TARGET");
$TARGET = '.';	# Keep current
}
}

if (!$CANCELLED)
{



if ($TARGET eq '')
{
$TARGET = ask_target();
}
my ($new_target) = SWGLO_validate( $TARGET);




if ($new_target ne ENV_getenv( 'GBS_TARGET'))
{
my $must_write = $new_target ne '' || $TARGET eq '-';
SWB_set( $new_target, $must_write);
}
ENV_sig( W => "No current Target")
if ($new_target eq '' && $TARGET ne '-');










{
my @lines;
if (ENV_get_changed_envs())
{
push @lines, GBSENV_changed_setenv_commands( 0);
}
push @lines, GBSCMD_command( 'gbsshow', '--brief');

GBSENV_write_result_script( \@lines, 0);
}
}

ENV_exit( $RC);




sub END
{
ENV_print_end_msg( 0);
}




sub ask_target()
{
my $target = '';




if (scalar @GBS_TARGETS == 0)
{
ENV_sig( EE => "No Target(s) defined - use 'swt --new' to define Targets");
} elsif (scalar @GBS_TARGETS == 1)
{
$target = $GBS_TARGETS[0];
ENV_say( 1, "Single Target '$target' taken by default");
} else
{



my $max_name = 0;
map { $max_name = length( $_) if (length( $_) > $max_name) } @GBS_TARGETS;

my @target_refs;


if ($GBS_SUBSYS eq '')
{



foreach my $target (@GBS_TARGETS)
{
push @target_refs, [ $target, '  ' ];
}
} elsif ($GBS_COMPONENT eq '')
{



foreach my $target (@GBS_TARGETS)
{
my $is_subsys = (GBSGLO_subsystem_does_build( $GBS_SUBSYS, $target)) ? '+' : '-';
push @target_refs, [ $target, "$is_subsys " ];
}
} else
{



foreach my $target (@GBS_TARGETS)
{
my $is_subsys = (GBSGLO_subsystem_does_build( $GBS_SUBSYS, $target)) ? '+' : '-';
my $is_component = (GBSGLO_component_does_build( $GBS_SUBSYS, $GBS_COMPONENT, $target)) ? '+' : '-';
push @target_refs, [ $target, "$is_subsys$is_component" ];
}
}

my @menu_refs = ( [ '', '-' ] );
foreach my $ref (@target_refs)
{
my ($target, $availability) = @{$ref};
my $plugin_name = SYSTEM_get_os_ref_value( $GBS_ROOT_PATH, '.', 'targets', $target);
my $string = sprintf( "%-*s [%s] [%s]", $max_name, $target, $availability, $plugin_name);
push @menu_refs, [ $string , $target ];
}

my $default_target = ENV_getenv( 'GBS_TARGET');
my $prompt = "Select Target ($GBS_SUBSYS,$GBS_COMPONENT)";
my $help   = '[<s><c>] -> s = Active for current Subsys, c = Active for current Component';
$target = ASK_value_from_menu( [ $prompt, $help ], \$default_target, @menu_refs);
$target = '-'
if ($target eq '');
}

return $target;
}


