#===============================================================
#
#   format.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package glo::format;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
FORMAT_cols
FORMAT_table
FORMAT_indent
FORMAT_hanging
);
}




use glo::env;




sub FORMAT_cols($$$$$);
sub FORMAT_table($$$$@);
sub FORMAT_indent($$@);
sub FORMAT_hanging($$$$);

sub get_width($);








sub FORMAT_cols($$$$$)
{
my ($width,			# 0 = term-width, <0 = term_width - $width, undef == anylength
$l_margin,		# size
$separator,		# string
$min_rows,		# minimum number of rows
$items_ref,		# [ @items ]
) = @_;
my @out_lines;

$width = get_width( $width);
my $table_width = $width - $l_margin;
my $separator_l = length $separator;
my $nr_items = scalar @{$items_ref};




my $max_item_l = 1;
map { $max_item_l = length $_ if (length $_ > $max_item_l) } @{$items_ref};




my $nr_cols = ($table_width + $separator_l) / ($max_item_l + $separator_l);
$nr_cols = 1
if ($nr_cols < 1);
my $nr_rows = ($nr_items + $nr_cols - 1) / $nr_cols;



if ($nr_rows <= $min_rows)
{
$nr_cols = $nr_items / $min_rows;
$nr_cols = 1
if ($nr_cols < 1);
$nr_rows = ($nr_items + $nr_cols - 1) / $nr_cols;



}





my $l_margin_spaces = ' ' x $l_margin;
if ($nr_cols > 0)
{
for (my $r = 0; $r < $nr_rows; $r++)
{
my $out = $l_margin_spaces;
for (my $c = 0; $c < $nr_cols; $c++)
{
my $item = $items_ref->[$r + $nr_rows * $c];
$out .= $separator if ($c != 0);
$item = ''
if (!defined $item);
$out .= sprintf "%-${max_item_l}s", $item;
}
push @out_lines, $out;
}
} else
{
@out_lines = map { "$l_margin_spaces$_" } @{$items_ref};
}

return @out_lines;
}





sub FORMAT_table($$$$@)
{
my ($width,			# 0 = term-width, <0 = term_width - $width, undef == anylength
$l_margin,		# size
$separator,		# string
$formats_ref,		# May be undef. Formats: L R
@row_refs,		# rows that are undef are left (and generated) undef
) = @_;
my @out_lines;

return () if (@row_refs == 0);

$width = get_width( $width);
my $table_width = $width - $l_margin;
my $separator_l = length $separator;




my @widths;
foreach my $ref (@row_refs)
{
if (defined $ref)
{
my $i = 0;
foreach my $cell (@{$ref})
{
my $length = length $cell;
if (defined $widths[$i])
{
$widths[$i] = $length
if ($length > $widths[$i]);
} else
{
$widths[$i] = $length;
}
$i++;
}
$ref = [ @{$ref} ];	# copy the row, because we are going to modify it
}
}
my $nr_cols = scalar @widths;
my $last_col = $nr_cols - 1;


my @formats = (defined $formats_ref) ? @{$formats_ref} : map { 'L' } 0..$last_col;
if (@formats < $nr_cols)
{
while (@formats < $nr_cols) { push @formats, 'L'; }
}





my $new_table_width = -$separator_l;
map { $new_table_width += ($_ + $separator_l) } @widths;

if ($new_table_width > $table_width)
{
my $width_minus_last = $new_table_width - ($widths[$last_col]);
my $last_width = $table_width - $width_minus_last;
$widths[$last_col] = $last_width if ($last_width > 3);
}





foreach my $ref (@row_refs)
{
if (defined $ref)
{
my $overflow;
do
{
$overflow = 0;
my $line = ' ' x $l_margin;
for (my $i = 0; $i < $nr_cols; $i++)
{
my $item = $ref->[$i];
$item = ''
if (!defined $item);
my $col_width = $widths[$i];
my $item_l = length $item;
if ($item ne '')
{
if ($item_l > $col_width)
{

$overflow = 1;
(my $indent) = $item =~ /^(\s*)/;   # get nr leading spaces
my $indent_l = length $indent;
my $ri = rindex( substr( $item, $indent_l, $col_width), ' ');

if ($ri >= 0)
{
$ref->[$i] = $indent . substr( $item, $ri + 1);
$item = substr( $item, 0, $ri);
} else
{
$ref->[$i] = $indent. substr( $item, $col_width);
$item = substr( $item, 0, $col_width);
}
} else
{
$ref->[$i] = '';
}
}
if ($formats[$i] eq 'L')
{
$line .= sprintf "%-${col_width}.${item_l}s", $item;	    # max_with.width
} else
{
$line .= sprintf "%${col_width}.${item_l}s", $item;	    # max_with.width
}
$line .= $separator unless ($i == $last_col);
}
$line =~ s/\s+$//;	    #remove trailing white-space
push @out_lines, $line;
} while ($overflow);
} else
{
push @out_lines, undef;
}
}

return @out_lines;
}




sub FORMAT_indent($$@)
{
my ($width,			# 0 = term-width, <0 = term_width - $width, undef == anylength
$l_margin,		# size
@row_refs,		# [ $indent, $text ]
) = @_;
my @out_lines;

return () if (@row_refs == 0);

$width = get_width( $width);
my $usable_width = $width - $l_margin;

foreach my $ref (@row_refs)
{
my ($indent, $text) = @{$ref};
my $lead = ' ' x ($l_margin + $indent);
my $col_width = $usable_width - $l_margin - $indent;
my $line;
while (length( $text) > 0)
{
if (length($text) > $col_width)
{
my $part;
my $ri = rindex( $text, ' ', $col_width);
if ($ri >= 0)
{
$part = substr( $text, 0, $ri);
$text = substr( $text, $ri + 1);
} else
{
$part= substr( $text, 0, $col_width);
$text = substr( $text, $col_width);
}
$line = $lead . $part;
} else
{
$line = $lead . $text;
$text = '';
}
push @out_lines, $line;
}
}

return @out_lines;
}




sub FORMAT_hanging($$$$)
{
my ($width,			# 0 = term-width, <0 = term_width - $width, undef == anylength
$l_margin,		# size
$indent,
$text,
) = @_;
my @out_lines;

$width = get_width( $width);
my $usable_width = $width - $l_margin;
if (length( $text) < $usable_width)
{
push @out_lines, (' ' x $l_margin) . $text;
} else
{
my $lead = ' ' x $l_margin;
my $col_width = $usable_width - $l_margin;
my $line;
while (length( $text) > 0)
{
if (length($text) > $col_width)
{
my $part;
my $ri = rindex( $text, ' ', $col_width);
if ($ri >= 0)
{
$part = substr( $text, 0, $ri);
$text = substr( $text, $ri + 1);
} else
{
$part= substr( $text, 0, $col_width);
$text = substr( $text, $col_width);
}
$line = $lead . $part;
} else
{
$line = $lead . $text;
$text = '';
}
push @out_lines, $line;
$lead = ' ' x ($l_margin + $indent);
$col_width = $usable_width - $l_margin - $indent;
}
}

return @out_lines;
}




sub get_width($)
{
my ($width) = @_;

if (defined $width)
{
if ($width <= 0)
{
$width = ENV_get_term_size() + $width;  # negative so acutally subtract!
}
$width--;   # cater for 1/2 characters
} else
{
$width = 1000;		       #(temp)
}

return $width;
}

1;
