#===============================================================
#
#   parse.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package glo::parse;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
PARSE_c_nc
PARSE_c_get_includes
);
}








sub PARSE_c_nc($);
sub PARSE_c_get_includes($);








my $C_NC = qr!/\*[^*]*\*+([^/*][^*]*\*+)*/|//[^\n]*|("(\\.|[^"\\])*"|'(\\.|[^'\\])*'|.[^/"'\\]*)!s;























sub PARSE_c_nc($)
{
my ($lines_ref) = @_;	# In/Out Single string!

no warnings;    # Use of uninitialized value $2 in substitution iterator at ...
$$lines_ref =~ s!$C_NC!$2!gs;
}




sub PARSE_c_get_includes($)
{
my ($lines_ref) = @_;	# In
my @include_refs;





my $line_nr = 1;
my $lines = $$lines_ref;
$lines =~ s/^/$line_nr++ . "\t"/gme;    # global, multi-line, expression





{
no warnings;
$lines =~ s!$C_NC!$2!gs;		    # global, single-line
}




my @include_lines = $lines =~ /^(\d+\t\s*#\s*include\s+.*[>"])/gm;    # global, multi-line





foreach my $include_line (@include_lines)
{
my ($line_nr, $inc_type, $inc_spec) = $include_line =~ /^(\d+)\t.*([<"])\s*(.*)\s*[>"]/;
push @include_refs, [ $line_nr, $inc_type, $inc_spec ];

}

return @include_refs;
}

1;


