#===============================================================
#
#   rootsglo.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package rootsglo;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
ROOTSGLO_print_root_cols
ROOTSGLO_get_helpline
ROOTSGLO_format_root_paths
);
}




use glo::env;
use glo::format;
use swglo;
use system;
use roots;




sub ROOTSGLO_print_root_cols();
sub ROOTSGLO_get_helpline();
sub ROOTSGLO_format_root_paths($);








my $HELP_LINE = '*=Other Platform, +=Multiple Platforms, x=Does not exist, ?=Invalid - Not GBS';








sub ROOTSGLO_print_root_cols()
{
ENV_say( 2, $HELP_LINE);
ENV_say( 0, FORMAT_cols( 0, 2, '  ', 2, [ ROOTSGLO_format_root_paths( 0) ]));
}




sub ROOTSGLO_get_helpline()
{
return $HELP_LINE;
}




sub ROOTSGLO_format_root_paths($)
{
my ($want_refs) = @_;
my @root_path_refs;

my @menu_texts;




my @row_refs;

foreach my $root_path (ROOTS_get_all( 1))	# $perl_format
{
my $menu_text = '';
my $fc = 'x';		    # Missing
my $system_name = '';
my $root_version = '';
if (-e $root_path)
{
$fc = '?';		# Not GBS Root (invalid)
if (SWGLO_check_system( $root_path))
{
$fc = ' ';		# Valid GBS Root
}
if (SWGLO_check_system_other_platform( $root_path))
{
$fc = ($fc eq ' ') ? '+' : '*';	# + = Both platfoms, * = Only Other platform
}
if ($fc ne '?')
{
$root_version = eval { SYSTEM_get( $root_path, 'root_version') };
if (!defined $root_version)
{
$root_version = '';
$fc = '?';
}
$system_name = eval { SYSTEM_get( $root_path, 'system_name') };
if (!defined $system_name)
{
$system_name = '';
$fc = '?';
}
}
}
push @root_path_refs, [ $menu_text, $root_path, $fc, $system_name, $root_version ];
push @row_refs, [ $fc, $system_name, $root_path, "[$root_version]" ];
}

@menu_texts = FORMAT_table( undef, 0, ' ', undef, @row_refs);
if ($want_refs)
{
map { $_->[0] = shift @menu_texts } @root_path_refs;

return @root_path_refs;
} else
{
return @menu_texts;
}
}

1;
