#===============================================================
#
#   scaglo.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package scaglo;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SCAGLO_init
SCAGLO_validate
SCAGLO_create_msg_and_help_files
SCAGLO_handle_unkn_msg
);
}




use glo::env;
use gbsenv;
use scamsg;
use scahelp;




sub SCAGLO_init($$$);
sub SCAGLO_validate(@);
sub SCAGLO_create_msg_and_help_files();
sub SCAGLO_handle_unkn_msg($$$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();

my $GBS_SILO_CACHE_PATH = "$GBS_ROOT_PATH/silo/.gbs/cache";




my $PLUGIN_NAME;
my $PLUGIN_REL;
my $PLUGIN_NICE_NAME;

my $GBS_SILO_PLUGIN_PATH;   # $GBS_SILO_CACHE_PATH/$PLUGIN_NAME/$PLUGIN_REL
my $GBS_MSG_FILESPEC;	    # $GBS_SILO_PLUGIN_PATH/msg.txt




my $UNKN_MSG_FUNC;
my $PLUGIN_MSG_FILESPECS_REF;
my $PLUGIN_READ_MSG_FILE_FUNC;
my $HELP_FILE_REQUIRED;	    	# Bool




sub SCAGLO_init($$$)
{
($PLUGIN_NAME,
$PLUGIN_REL,
$PLUGIN_NICE_NAME,
) = @_;

$GBS_SILO_PLUGIN_PATH = "$GBS_SILO_CACHE_PATH/$PLUGIN_NAME/$PLUGIN_REL";




$GBS_MSG_FILESPEC = SCAMSG_init( $PLUGIN_NAME, $PLUGIN_REL);
}






sub SCAGLO_validate(@)
{
($UNKN_MSG_FUNC,
$PLUGIN_MSG_FILESPECS_REF,
$PLUGIN_READ_MSG_FILE_FUNC,
$HELP_FILE_REQUIRED,		# Bool
) = @_;

my $must_create_all = 0;




if (! -e $GBS_SILO_PLUGIN_PATH)
{
ENV_say( 1, "Creating Silo Plugin dirs...");
mkdir "$GBS_SILO_CACHE_PATH/$PLUGIN_NAME";		    # should already be there
mkdir "$GBS_SILO_CACHE_PATH/$PLUGIN_NAME/$PLUGIN_REL";
$must_create_all = 1;
}

my $gbs_help_filespec = SCAHELP_get_filespec( $PLUGIN_NAME, $PLUGIN_REL);




if (!$must_create_all)
{



foreach my $plugin_msg_filespec (@{$PLUGIN_MSG_FILESPECS_REF})
{
if (ENV_file_is_newer( $plugin_msg_filespec, $GBS_MSG_FILESPEC))
{
$must_create_all = 1;
last;
}
if ($HELP_FILE_REQUIRED && !$must_create_all)
{



if (ENV_file_is_newer( $plugin_msg_filespec, $gbs_help_filespec))
{
$must_create_all = 1;
last;
}
}
}
}

if ($must_create_all)
{
SCAGLO_create_msg_and_help_files();
}
}




sub SCAGLO_create_msg_and_help_files()
{



if ($HELP_FILE_REQUIRED)
{
ENV_say( 1, "Creating msg and help files...");
} else
{
ENV_say( 1, "Creating msg file...");
}

SCAHELP_init( $PLUGIN_NAME, $PLUGIN_REL, $PLUGIN_NICE_NAME)
if ($HELP_FILE_REQUIRED);

foreach my $msg_ref ($PLUGIN_READ_MSG_FILE_FUNC->( $HELP_FILE_REQUIRED))	    # All info
{
if (defined $msg_ref->[0])
{



my ($msg_nr, $level, $level_text, $msg_text, $help_file_url, $heading, $lines_ref) = @{$msg_ref};

SCAMSG_add_msg( [ $msg_nr, $level, $level_text, $msg_text, $help_file_url ]);
SCAHELP_add_msg_text( $msg_nr, $level_text, $heading, $lines_ref )
if ($HELP_FILE_REQUIRED);
} else
{



if ($HELP_FILE_REQUIRED)
{
my (undef, $heading, $lines_ref) = @{$msg_ref};
SCAHELP_add_general_text( $heading, $lines_ref);
}
}
}




SCAMSG_write_file( 1);	# force_write
SCAHELP_write_help_file()
if ($HELP_FILE_REQUIRED);
}




sub SCAGLO_handle_unkn_msg($$$)
{


return $UNKN_MSG_FUNC->( @_);
}

1;

