#===============================================================
#
#   scahelp.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package scahelp;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SCAHELP_get_filespec
SCAHELP_init
SCAHELP_add_msg_text
SCAHELP_add_general_text
SCAHELP_write_help_file
);
}




use glo::env;
use glo::spit;
use glo::file;
use glo::html;
use gbshtml;
use gbsenv;
use scahelpglo;




sub SCAHELP_get_filespec($$);
sub SCAHELP_init($$$);
sub SCAHELP_add_msg_text($$$$);
sub SCAHELP_add_general_text($$);
sub SCAHELP_write_help_file();




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();

my $GBS_SILO_CACHE_PATH = "$GBS_ROOT_PATH/silo/.gbs/cache";




my $PLUGIN_NAME;
my $PLUGIN_REL;
my $PLUGIN_NICE_NAME;

my $GBS_SILO_PLUGIN_PATH;   # $GBS_SILO_CACHE_PATH/$PLUGIN_NAME/$PLUGIN_REL

my $HELP_FILESPEC;	    # $GBS_SILO_PLUGIN_PATH/msg_help.html

my $TITLE;
my @HEADING_LINES;
my @HTML_LINES;
my @HTML_HEADINGS;
my $NEXT_SECTION = 0;




sub SCAHELP_get_filespec($$)
{
my ($plugin_name,
$plugin_rel,
) = @_;


return "$GBS_SILO_CACHE_PATH/$plugin_name/$plugin_rel/" . SCAHELPGLO_get_file();
}




sub SCAHELP_init($$$)
{
($PLUGIN_NAME,
$PLUGIN_REL,
$PLUGIN_NICE_NAME,
) = @_;

$GBS_SILO_PLUGIN_PATH = "$GBS_SILO_CACHE_PATH/$PLUGIN_NAME/$PLUGIN_REL";
$HELP_FILESPEC = "$GBS_SILO_PLUGIN_PATH/" . SCAHELPGLO_get_file();

$TITLE = "$PLUGIN_NICE_NAME ($PLUGIN_REL) Warnings Helpfile";

GBSHTML_init( 1);	    # Use Silo




my $title = "$TITLE - Created by GBS";
push @HEADING_LINES, GBSHTML_doc_start( $HELP_FILESPEC, $title, 0, '', undef); # $want_scripts, $body_class, $target_frame_name
push @HEADING_LINES, GBSHTML_doc_top( $TITLE, ());
$NEXT_SECTION = 2;	    # Section 1 == Index
push @HEADING_LINES, GBSHTML_h( 1, undef, 'Contents', $NEXT_SECTION);
}




sub SCAHELP_add_msg_text($$$$)
{
my ($msg_nr,
$level_text,
$heading,
$lines_ref,
) = @_;

my $id = SCAHELPGLO_get_msg_id( $msg_nr);
push @HTML_LINES, GBSHTML_h( 2, { id => $id }, "$msg_nr ($level_text) -- $heading", $NEXT_SECTION);
push @HTML_LINES, HTML_pre_s();
my $line = join( "\n", @{$lines_ref});
push @HTML_LINES, HTML_pre_txt( $line);
push @HTML_LINES, HTML_pre_e();

}



sub SCAHELP_add_general_text($$)
{
my ($heading,
$lines_ref,
) = @_;

push @HTML_HEADINGS, $heading;

$NEXT_SECTION++;
push @HTML_LINES, GBSHTML_h( 1, undef, $heading, $NEXT_SECTION);
push @HTML_LINES, HTML_pre_s();
my $line = join( "\n", @{$lines_ref});
push @HTML_LINES, HTML_pre_txt( $line);
push @HTML_LINES, HTML_pre_e();
}





sub SCAHELP_write_help_file()
{



push @HEADING_LINES, GBSHTML_get_toc(1);    # $deepest_level




my @tailing_lines;
push @tailing_lines, GBSHTML_doc_bottom();
push @tailing_lines, GBSHTML_doc_end();




ENV_say( 1, "Writing SCA Help file...");
my $fh = FILE_lock_ex_wait( "$HELP_FILESPEC.lck");
SPIT_file_nl( $HELP_FILESPEC, \@HEADING_LINES, \@HTML_LINES, \@tailing_lines);
FILE_unlock( $fh);

@HTML_HEADINGS = ();
@HTML_LINES = ();
$TITLE = undef;

return $HELP_FILESPEC;
}

1;

