#===============================================================
#
#   setup.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package setup;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SETUP_ask_site
SETUP_ask_log_root
SETUP_show_licence
);
}




use glo::env;
use glo::ask;
use glo::slurp;





sub SETUP_ask_site($);
sub SETUP_ask_log_root($);
sub SETUP_show_licence();








sub SETUP_ask_site($)
{
my ($default_site,
) = @_;
my $new_site;

$default_site = 'HOME'
if ($default_site eq '');

ENV_say( 1, "Enter your Site-ID (pref. 3 letter IATA Airport Code)");
$new_site = ASK_word( 'Site-Id', $default_site, [ 3, 20 ]);

return $new_site;
}





sub SETUP_ask_log_root($)
{
my ($default_log_path,
) = @_;
my $new_log_path;

$default_log_path = ENV_get_application_home_path() . "/GBSLog"
if ($default_log_path eq '');

my $again;
do
{
$again = 0;
$new_log_path = ASK_path( 'LogRoot', $default_log_path, 0, -1, undef);
if (!-d $new_log_path)
{
ENV_say( 1, "Directory does not exist");
if (ASK_YN( 'Create?', 'Y') eq 'Y')
{
if (!mkdir( $new_log_path))
{
ENV_beep();
ENV_say( 2, "- Unable to create '$new_log_path'",
"- $!");
$again = 1;
}
} else
{
ENV_beep();
ENV_say( 2, "- Directory must exist");
$again = 1;
}
}
} while ($again);

return ENV_os_path( $new_log_path);
}




sub SETUP_show_licence()
{
my $gbs_scripts = ENV_getenv_perl_path( 'GBS_SCRIPTS_PATH');
ENV_say( 0, '');
ENV_say( 1, "*** Welcome to GBS ***",
'');
map { ENV_say( 0, "  $_"); } SLURP_file( "$gbs_scripts/licence.txt");
ENV_say( 0, '');
if (ASK_YN( 'I agree with the above', 'N') eq 'N')
{
ENV_say( 1, "Cannot proceed");
ENV_exit( 1);
}
ENV_say( 0, '');
}

1;


