#!/bin/bash
#=========================================
#  define.bat <rel> <command> [<command_arg> ...]
#   Called by a GBS shortcut
#   - Set GBS_BASE_PATH
#   - Call $GBS_BASE_PATH/setup.sh
#   - execute the gbscmd - if present
#   - Args:
#
#     <rel>:     'cur', 'beta' or actual version
#     <command>: 'gbs', 'perl', 'source' or 'exec'
#=========================================

#
#   Handle Args
#
GBS_DEFINE_REL=$1; shift
GBS_DEFINE_COMMAND=$1; shift
GBS_DEFINE_COMMAND_ARGS=$*
set --
[[ $GBS_DEFINE_COMMAND == "" ]] && GBS_DEFINE_COMMAND=gbs

#
#   Set GBS_BASE_PATH
#
\export GBS_BASE_PATH="$HOME/.gbs"
#\echo "DEFINE_: GBS_BASE_PATH=$GBS_BASE_PATH"

#
#   Assert TMPDIR
#
if [[ ! -z TMPDIR ]]
then
    \export "TMPDIR=/tmp/$USER"
    \mkdir "$TMPDIR" 2>/dev/null
fi

#
#   Execute gbscmd.sh if exist
#
\export GBS_LOGIN_FILE=$GBS_BASE_PATH/gbscmd.sh
if [ -e $GBS_LOGIN_FILE ]
then
    \echo DEFINE_: Executing $GBS_LOGIN_FILE...
    \source $GBS_LOGIN_FILE
fi
unset GBS_LOGIN_FILE

if [[ $GBS_DEFINE_COMMAND == "gbs"  ]]
then
    #
    #   Setup init_rel file
    #
    if [[ $GBS_DEFINE_REL != "" ]]
    then
	\echo "DEFINE_: GBS version set to [$GBS_DEFINE_REL]"
    fi
    \echo "GBS_WANTED_REL=$GBS_DEFINE_REL" > "$TMPDIR/gbsinit_rel_$PPID.sh"

    #
    #   Assert alias gbs
    #
    GBS_DEFINED=0
    if [[ $(grep -E "^\s*\\\?alias\s+gbs=" ~/.bashrc) || \
	  ( -f ~/.bash_aliases && $(grep -E "^\s*\\\?alias\s+gbs=" ~/.bash_aliases) ) ]]
    then
	\echo "DEFINE_: GBS commands defined:"
	\echo "         gbs"
	\echo "         gbshelp"
	\echo "         gbssetup"
	\echo "         gbsuninstall"
	\echo "-"
	\echo "DEFINE_: Type 'gbs' to startup GBS [$GBS_DEFINE_REL]"
    else
	\echo "DEFINE_: gbs alias not defined in ~/.bashrc or ~/.bash_aliases"
	\echo "         Type 'source ~/.gbs/gbsinit.sh' to startup GBS [$GBS_DEFINE_REL]"
    fi
else
    #
    #   Run setup (& profile)
    #
    \source "$GBS_BASE_PATH/setup.sh" $GBS_DEFINE_REL

    #
    #  Execute the specified command
    #
    \echo "DEFINE_: Executing $GBS_DEFINE_COMMAND $GBS_DEFINE_COMMAND_ARGS..."
    if [[ $GBS_DEFINE_COMMAND == perl ]]
    then
        $GBS_PERL_CMD $GBS_SCRIPTS_PATH/$GBS_DEFINE_COMMAND_ARGS
    elif [[ $GBS_DEFINE_COMMAND == source ]]
    then
        \source $GBS_SCRIPTS_PATH/$GBS_DEFINE_COMMAND_ARGS
    elif [[ $GBS_DEFINE_COMMAND == exec ]]
    then
        $GBS_SCRIPTS_PATH/$GBS_DEFINE_COMMAND_ARGS
    else
	\echo "DEFINE_: **ERROR** Invalid command '$GBS_DEFINE_COMMAND' (perl source exec)"
	\echo "         define $GBS_DEFINE_REL $GBS_DEFINE_COMMAND $GBS_DEFINE_COMMAND_ARGS"
    fi
fi

#
#   Cleanup
#
unset GBS_DEFINE_REL
unset GBS_DEFINE_COMMAND
unset GBS_DEFINE_COMMAND_ARGS

#   End

#  **EOF**
