#==============================================================
#
#   swb.pm
#
#===============================================================
#   This file is a part of the GBS (Generic Build Support)
#   package created by Randy Marques, Netherlands
#   For information: http://www.genericbuildsupport.com
#   Copyright  2010-2020 - Randy Marques - All rights reserved
#===============================================================
package swb;

use strict;
use warnings;
use integer;

BEGIN
{
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
SWB_validate_dirs
SWB_set
);
}




use glo::env;
use gbsenv;
use gbsglo;
use gbsrc;
use system;




sub SWB_validate_dirs($$$$);
sub SWB_set($$);




my $GBS_ROOT_PATH = GBSENV_gbs_root_path();




sub SWB_validate_dirs($$$$)
{
my ($root_path,
$subsys,
$component,
$target,
) = @_;
my $new_target = $target;


if ($subsys ne '')
{



my $gen_dir = "$root_path/dev/$subsys/build";
if (-d "$gen_dir/$target")
{



if ($component ne '' && GBSGLO_subsystem_is_full_gbs( $subsys, $root_path))
{
my $bld_dir = "$root_path/dev/$subsys/comp/$component/bld";
if (!-d "$bld_dir/$target")
{
ENV_sig( W => "Component '$component' does not generate for Target '$target'");
$new_target = '';
}
}
} else
{
ENV_sig( W => "SubSystem '$subsys' does not generate for Target '$target'");
$new_target = '';
}
}

return $new_target;
}




sub SWB_set($$)
{
my ($target,
$must_write,
) = @_;

ENV_debug( 1, "Set $GBS_ROOT_PATH ($target)");




my $target_plugin = ($target eq '') ? '' : SYSTEM_get_os_ref_value( $GBS_ROOT_PATH, '.', 'targets', $target);
ENV_setenv( GBS_TARGET => $target);
ENV_setenv( GBS_TARGET_PLUGIN => $target_plugin);

if (GBSENV_mode_is_interactive())
{



GBSRC_write_root( target => $target)
if ($must_write);
}
}

1;


