#==============================================================================
#
#   [ %SUBSYS% ] Makefile.mk
#
#   make all | clean
#
#==============================================================================
#
#   Applicable EnvVars:
#       GBS_TARGET
#       GBS_TARGET_PLUGIN
#	GBS_TARGET_PLUGIN_PATH
#	GBS_TARGET_PLUGIN_BIN_PATH
#
#   Availabe EnvVars:
#       GBS_ROOT_PATH
#       GBS_SUBSYS
#       GBS_SUBSYS_PATH
#       GBS_BUILD_PATH
#       GBS_APP_PATH
#
#       GBS_IGNORE_ERRORS       0 | 1
#       GBS_MODE                FINAL | ASSERT | DEBUG | PROFILING
#       GBS_OPT                 YES | SPEED | SIZE | DEBUG | NO
#       GBS_DEBUGGER            NO | YES
#       GBS_MAP                 NO | YES
#
#       GBS_BLD_<src_type>      <primary_output_type>
#       GBS_SYSFLAGS_<src_type>
#       GBS_FLAGS_<src_type>    also contains flags resulting from MODE, OPT, DEBUGGER and MAP
#       GBS_SYSINCS_<src_type>
#       GBS_INCS_<src_type>
#==============================================================================

#==============================================================================
#   DEFINITIONS
#==============================================================================
#------------------------------------------------------------------------------
#   Build Commands
#------------------------------------------------------------------------------
#
#   Select Plugin bin path
#
PLUGIN_BIN_PATH = ${GBS_TARGET_PLUGIN_BIN_PATH}

CC = ${PLUGIN_BIN_PATH}gcc -c -Wall -ansi
CPP = ${PLUGIN_BIN_PATH}g++ -c -Wall
AS = ${PLUGIN_BIN_PATH}as --warn --fatal-warnings
AR = ${PLUGIN_BIN_PATH}ar -rs
LD = ${PLUGIN_BIN_PATH}gcc

#------------------------------------------------------------------------------
#   Directories
#------------------------------------------------------------------------------
SRC = app/src
BLD = app/bld/${GBS_TARGET}

#------------------------------------------------------------------------------
#   Includes
#------------------------------------------------------------------------------
C_INCLUDES = ${GBS_INCS_C} ${GBS_SYSINCS_C}
GLK_INCLUDES = ${GBS_INCS_GLK} ${GBS_SYSINCS_GLK}

#------------------------------------------------------------------------------
#   Flags
#------------------------------------------------------------------------------
C_FLAGS = ${GBS_SYSFLAGS_C} ${GBS_FLAGS_C}
GLK_FLAGS = ${GBS_SYSFLAGS_GLK} ${GBS_FLAGS_GLK}

#==============================================================================
#   RULES
#==============================================================================
#------------------------------------------------------------------------------
#   all
#------------------------------------------------------------------------------
all: ${BLD}/main${GBS_BLD_GLK}

#
#   Linking 'main'
#
MAIN_DEPEND = ${BLD}/main${GBS_BLD_C}
${BLD}/main${GBS_BLD_GLK}: ${MAIN_DEPEND}
	${LD} ${MAIN_DEPEND} ${GLK_INCLUDES} ${GLK_FLAGS} -o $@

#
#   Compiling
#
${BLD}/main${GBS_BLD_C}: ${SRC}/main.c
	${CC} ${SRC}/main.c ${C_INCLUDES} ${C_FLAGS} -o $@

#------------------------------------------------------------------------------
#   clean
#------------------------------------------------------------------------------
clean:
	touch ${BLD}/clean.up
	rm ${BLD}/*.*
	@echo "MAKEFILE: 'clean' done"

###EOF###
